package com.book.server.controller;

import com.book.dao.cps.pojo.SearchKeyword;
import com.book.server.common.entity.Result;
import com.book.server.common.entity.ResultCode;
import com.book.dao.VO.*;
import com.book.server.service.BookService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 书籍
 */
@RestController
@RequestMapping("/api/book")
@Slf4j
public class BookController extends BaseController {
    @Autowired
    private BookService bookService;


    /**
     * 书库查询
     *
     * @param queryVO
     * @return
     */
    @PostMapping("/query")
    public Result<List<BookRes>> query(@RequestBody QueryVO queryVO) {
        return Result.success(bookService.query(queryVO));
    }

    /**
     * 书城分页推荐-精彩推荐
     * novel_end_recommend
     *
     * @param queryVO
     * @return
     */
    @PostMapping("/recommend")
    public Result<List<BookRes>> recommendByRead(@RequestBody QueryVO queryVO) {
        return Result.success(bookService.recommendByRead(queryVO));
    }

    /**
     * 主编推荐
     * smart_recommand
     *
     * @param queryVO
     * @return
     */
    @PostMapping("/smartRecommand")
    public Result<List<BookRes>> smartRecommand(@RequestBody QueryVO queryVO) {
        return Result.success(bookService.smartRecommand(queryVO));
    }

    /**
     * 热门推荐4个
     *
     * @param queryVO
     * @return
     */
    @PostMapping("/topRecommend")
    public Result<List<BookRes>> topRecommend(@RequestBody QueryVO queryVO) {
        List<BookRes> books = bookService.topRecommend(queryVO);
        return Result.success(books);
    }

    /**
     * 书城block
     *
     * @param pageId
     * @return
     */
    @GetMapping("/getBlockByPageId")
    //@Cacheable(value = "block", key = "#pageId")
    public Result<List<BlockRes>> getBlockByPageId(Integer pageId) {
        List<BlockRes> list = bookService.getBlockByPageId(pageId);
        return Result.byObject(list);
    }

    /**
     * 更新block缓存
     *
     * @return
     */
    @GetMapping("/resetBlockCache")
    @CacheEvict(value = "block", allEntries = true)
    public Result resetBlockCache() {
        return Result.byObject("OK");
    }


    /**
     * 导入书库
     * 注意：此代码仅限拉取书籍使用，只能运行一次。
     *
     * @return
     */

    @GetMapping("/insertBooks")
    public Result insertBooks() {
        boolean open = false;
        if (open) {
            bookService.insertBooks();
        }
        return Result.byObject("OK");
    }

    /**
     * 获取章节列表
     *
     * @param bookContentVO
     * @return
     */
    @PostMapping("/chapterList")
    public Result<List<Chapter>> getChaptersByPageAndRow(@RequestBody BookContentVO bookContentVO) {

        List<Chapter> chapterList = bookService.getChaptersByBookId(bookContentVO.getBookId());
        if (CollectionUtils.isEmpty(chapterList)) {
            return Result.failure(ResultCode.FAIL.getCode(), "哎呀~没有数据啦~");
        } else {
            return Result.byObject(chapterList);
        }
    }

    /**
     * 获取章节书籍内容
     *
     * @param bookContentVO 书籍id,章节id
     * @return
     */
    @PostMapping("/content")
    public Result<BookContent> getBookContentByChapter(@RequestBody BookContentVO bookContentVO) {

        BookContent content = bookService.getContentByContentId(bookContentVO.getBookId(),
                bookContentVO.getContentId());
        if (content == null || "null".equals(content)) {
            return Result.failure(ResultCode.FAIL.getCode(), "哎呀~没有数据啦~");
        }

        return Result.byObject(content);
    }

    /**
     * 获取热门搜索
     *
     * @param sex
     * @return
     */
    @GetMapping("/getSearchKeyword")
    public Result<List<SearchKeyword>> getSearchKeywordBySex(String sex) {
        return Result.byObject(bookService.getSearchKeywordBySex(sex));
    }

}
